# Copyright (C) 2017-2019  Lars Wirzenius
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from .version import __version__, __version_info__
from .logging import setup_logging, log
from .store import (
    MemoryStore,
    Conflict,
)
from .persistent import (
    MemoryPersistentState,
    NotFound,
    Resource,
    resource_from_dict,
)
from .trans import TransactionalState
from .build import Build, WrongBuildStatusChange
from .buildgraph import (
    BuildGraph,
    ACTION_BLOCKED,
    ACTION_READY,
    ACTION_BUILDING,
    ACTION_DONE,
    ACTION_FAILED,
)
from .buildsm import (
    create_build_event,
    BuildStateMachine,
    BuildStartsEvent,
    NeedWorkEvent,
    PartialActionOutputEvent,
    ActionFinishedEvent,
    ActionFailedEvent,
    BUILD_TRIGGERED,
    BUILD_BUILDING,
    BUILD_NOTIFYING,
    BUILD_DONE,
    BUILD_FAILED,
)
from .exceptions import (
    BadUpdate,
    ExistsAlready,
    IckException,
    MethodNotAllowed,
    ClientIdMissing,
    ParametersMissing,
)
from .responses import (
    OK,
    bad_request,
    conflict,
    created,
    not_found,
    text_plain,
)
from .apibase import APIbase, ResourceApiBase
from .buildsapi import BuildsAPI
from .logapi import LogAPI
from .versionapi import VersionAPI
from .pipelineapi import (PipelineAPI, NoWhere)
from .projectapi import ProjectAPI
from .workapi import WorkAPI
from .workerapi import WorkerAPI
from .controllerapi import (
    ControllerAPI,
)
from .notificationapi import NotificationAPI
from .blobapi import BlobAPI
from .blob_store import BlobStore

from .client import (
    HttpAPI,
    HttpError,
    ControllerClient,
    BlobClient,
    AuthClient,
    Reporter,
)
from .actionenvs import (
    Runner,
    ActionEnvironment,
    HostEnvironment,
    ChrootEnvironment,
    ContainerEnvironment,
)
from .workspace import WorkspaceArea, Workspace
from .actions import (
    ActionFactory,
    UnknownStepError,
    ShellAction,
    PythonAction,
    DebootstrapAction,
    CreateWorkspaceAction,
    ArchiveWorkspaceAction,
    PopulateSystreeAction,
    GitAction,
)

from .sendmail import Sendmail
